import arcade, arcade.gui, asyncio, pypresence, time, copy, json, os

from wizard_vs_irs.utils.preload import button_texture, button_hovered_texture
from wizard_vs_irs.utils.constants import big_button_style, discord_presence_id
from wizard_vs_irs.utils.utils import FakePyPresence

class Main(arcade.gui.UIView):
    def __init__(self, pypresence_client=None):
        super().__init__()

        self.anchor = self.add_widget(arcade.gui.UIAnchorLayout())
        self.box = self.anchor.add(arcade.gui.UIBoxLayout(space_between=10), anchor_x='center', anchor_y='center')

        self.pypresence_client = pypresence_client

        with open("settings.json", "r") as file:
            self.settings_dict = json.load(file)

        if self.settings_dict.get('discord_rpc', True):
            if self.pypresence_client == None: # Game has started
                try:
                    asyncio.get_event_loop()
                except:
                    asyncio.set_event_loop(asyncio.new_event_loop())
                try:
                    self.pypresence_client = pypresence.Presence(discord_presence_id)
                    self.pypresence_client.connect()
                    self.pypresence_client.start_time = time.time()
                except:
                    self.pypresence_client = FakePyPresence()
                    self.pypresence_client.start_time = time.time()

            elif isinstance(self.pypresence_client, FakePyPresence): # the user has enabled RPC in the settings in this session.
                # get start time from old object
                start_time = copy.deepcopy(self.pypresence_client.start_time)
                try:
                    self.pypresence_client = pypresence.Presence(discord_presence_id)
                    self.pypresence_client.connect()
                    self.pypresence_client.start_time = start_time
                except:
                    self.pypresence_client = FakePyPresence()
                    self.pypresence_client.start_time = start_time

            self.pypresence_client.update(state='In Menu', details='In Main Menu', start=self.pypresence_client.start_time)
        else: # game has started, but the user has disabled RPC in the settings.
            self.pypresence_client = FakePyPresence()
            self.pypresence_client.start_time = time.time()

        self.pypresence_client.update(state='In Menu', details='In Main Menu', start=self.pypresence_client.start_time)

        if os.path.exists("data.json"):
            with open("data.json", "r") as file:
                self.data = json.load(file)
        else:
            self.data = {}
        
        if not "high_score" in self.data:
            self.data["high_score"] = 0

        if not "evaded_tax" in self.data:
            self.data["evaded_tax"] = 0

        if not "shop" in self.data:
            self.data["shop"] = {}

        with open("data.json", "w") as file:
            file.write(json.dumps(self.data, indent=4))

    def on_show_view(self):
        super().on_show_view()

        self.title_label = self.box.add(arcade.gui.UILabel(text="Wizard vs IRS", font_name="Roboto", font_size=48))
        self.high_score_label = self.box.add(arcade.gui.UILabel(text=f"High Score: {self.data['high_score']}$", font_name="Roboto", font_size=24))
        self.evaded_tax_label = self.box.add(arcade.gui.UILabel(text=f"Total Evaded Tax: {self.data['evaded_tax']}$", font_name="Roboto", font_size=24))

        self.play_button = self.box.add(arcade.gui.UITextureButton(text="Play", texture=button_texture, texture_hovered=button_hovered_texture, width=self.window.width / 2, height=self.window.height / 10, style=big_button_style))
        self.play_button.on_click = lambda event: self.play()

        self.shop_button = self.box.add(arcade.gui.UITextureButton(text="Shop", texture=button_texture, texture_hovered=button_hovered_texture, width=self.window.width / 2, height=self.window.height / 10, style=big_button_style))
        self.shop_button.on_click = lambda event: self.shop()

        self.settings_button = self.box.add(arcade.gui.UITextureButton(text="Settings", texture=button_texture, texture_hovered=button_hovered_texture, width=self.window.width / 2, height=self.window.height / 10, style=big_button_style))
        self.settings_button.on_click = lambda event: self.settings()

    def play(self):
        from wizard_vs_irs.game.play import Game
        self.window.show_view(Game(self.pypresence_client))

    def shop(self):
        from wizard_vs_irs.menus.shop import Shop
        self.window.show_view(Shop(self.pypresence_client))

    def settings(self):
        from wizard_vs_irs.menus.settings import Settings
        self.window.show_view(Settings(self.pypresence_client))



import arcade, arcade.gui, os, json

from math import ceil

from wizard_vs_irs.utils.constants import button_style, SHOP_ITEMS
from wizard_vs_irs.utils.preload import button_texture, button_hovered_texture

class Shop(arcade.gui.UIView):
    def __init__(self, pypresence_client):
        super().__init__()

        self.pypresence_client = pypresence_client
    
        self.anchor = self.add_widget(arcade.gui.UIAnchorLayout(size_hint=(1, 1)))
        self.grid = self.anchor.add(arcade.gui.UIGridLayout(column_count=4, row_count=ceil(len(SHOP_ITEMS) / 4), horizontal_spacing=10, vertical_spacing=10), anchor_x="center", anchor_y="center")

        if os.path.exists("data.json"):
            with open("data.json", "r") as file:
                self.data = json.load(file)
        else:
            self.data = {}

        if not "shop" in self.data:
            self.data["shop"] = {}
    
        self.shop_buttons = []

    def main_exit(self):
        with open("data.json", "w") as file:
            file.write(json.dumps(self.data, indent=4))

        from wizard_vs_irs.menus.main import Main
        self.window.show_view(Main(self.pypresence_client))

    def on_show_view(self):
        super().on_show_view()

        self.back_button = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text='<--', style=button_style, width=100, height=50)
        self.back_button.on_click = lambda event: self.main_exit()
        self.anchor.add(self.back_button, anchor_x="left", anchor_y="top", align_x=5, align_y=-5)

        self.evaded_tax_label = self.anchor.add(arcade.gui.UILabel(f"Evaded Tax: {self.data['evaded_tax']}$", font_size=24), anchor_x="center", anchor_y="top")

        for n, shop_item in enumerate(SHOP_ITEMS):
            row, col = n // 4, n % 4

            if not shop_item[1] in self.data["shop"]:
                self.data["shop"][shop_item[1]] = 0

            upgrade_cost = (self.data["shop"][shop_item[1]] + 1) * shop_item[3]

            max_count = shop_item[2]

            notice_string = "\n(Also increases knockback)" if "DMG" in shop_item[0] else ""
            level_string = self.data["shop"][shop_item[1]] if self.data["shop"][shop_item[1]] < max_count else "Max"

            button = self.grid.add(arcade.gui.UITextureButton(
                text=f'{shop_item[0]}{notice_string}\nLevel: {level_string}\nUpgrade Cost: {upgrade_cost}$',
                texture=button_texture,
                texture_hovered=button_hovered_texture,
                style=button_style,
                width=self.window.width / 8,
                height=self.window.width / 8,
                multiline=True,
                align="center"
            ), row=row, column=col)

            self.shop_buttons.append(button)

            button.on_click = lambda event, n=n: self.buy_upgrade(n)

    def buy_upgrade(self, n):
        item_list = SHOP_ITEMS[n]
        json_name = item_list[1]
        max_count = item_list[2]

        current_level = self.data["shop"][json_name]
        cost = (current_level + 1) * item_list[3]

        if not self.data["evaded_tax"] >= cost:
            return
        
        if current_level >= item_list[2]:
            return

        self.data["shop"][json_name] += 1
        self.data["evaded_tax"] -= cost

        notice_string = "\n(Also increases knockback)" if "DMG" in item_list[0] else ""
        level_string = self.data["shop"][item_list[1]] if self.data["shop"][item_list[1]] < max_count else "Max"

        self.shop_buttons[n].text = f"{item_list[0]}{notice_string}\nLevel: {level_string}\nUpgrade Cost: {cost + item_list[3]}"
        self.evaded_tax_label.text = f"Evaded Tax: {self.data['evaded_tax']}$"

    def on_key_press(self, symbol, modifiers):
        if symbol == arcade.key.ESCAPE:
            self.main_exit()


import logging, arcade, arcade.gui, sys, traceback

from wizard_vs_irs.utils.constants import menu_background_color

import pyglet.info, pyglet.event

def dump_platform():
    import platform
    logging.debug(f'Platform: {platform.platform()}')
    logging.debug(f'Release: {platform.release()}')
    logging.debug(f'Machine: {platform.machine()}')
    logging.debug(f'Architecture: {platform.architecture()}')

def dump_gl(context=None):
    if context is not None:
        info = context.get_info()
    else:
        from pyglet.gl import gl_info as info
    logging.debug(f'gl_info.get_version(): {info.get_version()}')
    logging.debug(f'gl_info.get_vendor(): {info.get_vendor()}')
    logging.debug(f'gl_info.get_renderer(): {info.get_renderer()}')

def print_debug_info():
    logging.debug('########################## DEBUG INFO ##########################')
    logging.debug('')
    dump_platform()
    dump_gl()
    logging.debug('')
    logging.debug(f'Number of screens: {len(pyglet.display.get_display().get_screens())}')
    logging.debug('')
    for n, screen in enumerate(pyglet.display.get_display().get_screens()):
        logging.debug(f"Screen #{n+1}:")
        logging.debug(f'DPI: {screen.get_dpi()}')
        logging.debug(f'Scale: {screen.get_scale()}')
        logging.debug(f'Size: {screen.width}, {screen.height}')
        logging.debug(f'Position: {screen.x}, {screen.y}')
    logging.debug('')
    logging.debug('########################## DEBUG INFO ##########################')
    logging.debug('')

class ErrorView(arcade.gui.UIView):
    def __init__(self, message, title):
        super().__init__()

        self.message = message
        self.title = title

    def exit(self):
        logging.fatal('Exited with error code 1.')
        sys.exit(1)

    def on_show_view(self):
        super().on_show_view()

        self.window.set_caption('Wizard vs IRS - Error')
        self.window.set_mouse_visible(True)
        self.window.set_exclusive_mouse(False)
        arcade.set_background_color(menu_background_color)

        msgbox = arcade.gui.UIMessageBox(width=self.window.width / 2, height=self.window.height / 2, message_text=self.message, title=self.title)
        msgbox.on_action = lambda _: self.exit()
        self.add_widget(msgbox)

def on_exception(*exc_info):
    logging.error(f"Unhandled exception:\n{''.join(traceback.format_exception(exc_info[1], limit=None))}")

def get_closest_resolution():
    allowed_resolutions = [(1366, 768), (1440, 900), (1600,900), (1920,1080), (2560,1440), (3840,2160)]
    screen_width, screen_height = arcade.get_screens()[0].width, arcade.get_screens()[0].height
    if (screen_width, screen_height) in allowed_resolutions:
        if not allowed_resolutions.index((screen_width, screen_height)) == 0:
            closest_resolution = allowed_resolutions[allowed_resolutions.index((screen_width, screen_height))-1]
        else:
            closest_resolution = (screen_width, screen_height)
    else:
        target_width, target_height = screen_width // 2, screen_height // 2

        closest_resolution = min(
            allowed_resolutions,
            key=lambda res: abs(res[0] - target_width) + abs(res[1] - target_height)
        )
    return closest_resolution

class FakePyPresence():
    def __init__(self):
        ...
    def update(self, *args, **kwargs):
        ...
    def close(self, *args, **kwargs):
        ...



import pyglet

pyglet.options.debug_gl = False

import logging, datetime, os, json, sys, arcade

from wizard_vs_irs.utils.utils import get_closest_resolution, print_debug_info, on_exception
from wizard_vs_irs.utils.constants import log_dir, menu_background_color
from wizard_vs_irs.menus.main import Main

def main():
    sys.excepthook = on_exception

    pyglet.resource.path.append(os.getcwd())
    pyglet.font.add_directory('./assets/fonts')

    if not log_dir in os.listdir():
        os.makedirs(log_dir)

    while len(os.listdir(log_dir)) >= 5:
        files = [(file, os.path.getctime(os.path.join(log_dir, file))) for file in os.listdir(log_dir)]
        oldest_file = sorted(files, key=lambda x: x[1])[0][0]
        os.remove(os.path.join(log_dir, oldest_file))

    timestamp = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    log_filename = f"debug_{timestamp}.log"
    logging.basicConfig(filename=f'{os.path.join(log_dir, log_filename)}', format='%(asctime)s %(name)s %(levelname)s: %(message)s', level=logging.DEBUG)

    for logger_name_to_disable in ['arcade', "numba"]:
        logging.getLogger(logger_name_to_disable).propagate = False
        logging.getLogger(logger_name_to_disable).disabled = True

    if os.path.exists('settings.json'):
        with open('settings.json', 'r') as settings_file:
            settings = json.load(settings_file)

        resolution = list(map(int, settings['resolution'].split('x')))

        if not settings.get("anti_aliasing", "4x MSAA") == "None":
            antialiasing = int(settings.get("anti_aliasing", "4x MSAA").split('x')[0])
        else:
            antialiasing = 0

        fullscreen = settings['window_mode'] == 'Fullscreen'
        style = arcade.Window.WINDOW_STYLE_BORDERLESS if settings['window_mode'] == 'borderless' else arcade.Window.WINDOW_STYLE_DEFAULT
        vsync = settings['vsync']
        fps_limit = settings['fps_limit']
    else:
        resolution = get_closest_resolution()
        antialiasing = 4
        fullscreen = False
        style = arcade.Window.WINDOW_STYLE_DEFAULT
        vsync = True
        fps_limit = 0

        settings = {
            "resolution": f"{resolution[0]}x{resolution[1]}",
            "antialiasing": "4x MSAA",
            "window_mode": "Windowed",
            "vsync": True,
            "fps_limit": 60,
            "discord_rpc": True
        }

        with open("settings.json", "w") as file:
            file.write(json.dumps(settings))

    window = arcade.Window(width=resolution[0], height=resolution[1], title='Wizard vs IRS', samples=antialiasing, antialiasing=antialiasing > 0, fullscreen=fullscreen, vsync=vsync, resizable=False, style=style)

    if vsync:
        window.set_vsync(True)
        display_mode = window.display.get_default_screen().get_mode()
        refresh_rate = display_mode.rate
        window.set_update_rate(1 / refresh_rate)
        window.set_draw_rate(1 / refresh_rate)
    elif not fps_limit == 0:
        window.set_update_rate(1 / fps_limit)
        window.set_draw_rate(1 / fps_limit)
    else:
        window.set_update_rate(1 / 99999999)
        window.set_draw_rate(1 / 99999999)

    arcade.set_background_color(menu_background_color)

    print_debug_info()
    main = Main()

    window.show_view(main)

    logging.debug('Game started.')

    arcade.run()

    logging.info('Exited with error code 0.')

if __name__ == "__main__":
    main()


import copy, pypresence, json

import arcade, arcade.gui

from wizard_vs_irs.utils.constants import button_style, dropdown_style, slider_style, settings, discord_presence_id, settings_start_category
from wizard_vs_irs.utils.utils import FakePyPresence
from wizard_vs_irs.utils.preload import button_texture, button_hovered_texture

from arcade.gui import UIBoxLayout, UIAnchorLayout

class Settings(arcade.gui.UIView):
    def __init__(self, pypresence_client, *args):
        super().__init__()

        self.args = args

        with open("settings.json", "r") as file:
            self.settings_dict = json.load(file)

        self.pypresence_client = pypresence_client
        self.pypresence_client.update(state='In Settings', details='Modifying Settings', start=self.pypresence_client.start_time)

        self.slider_labels = {}
        self.sliders = {}

        self.on_radiobuttons = {}
        self.off_radiobuttons = {}

        self.current_category = settings_start_category

        self.modified_settings = {}

    def create_layouts(self):
        self.anchor = self.add_widget(UIAnchorLayout(size_hint=(1, 1)))

        self.box = UIBoxLayout(space_between=50, align="center", vertical=False)
        self.anchor.add(self.box, anchor_x="center", anchor_y="top", align_x=10, align_y=-75)

        self.top_box = UIBoxLayout(space_between=self.window.width / 160, vertical=False)
        self.anchor.add(self.top_box, anchor_x="left", anchor_y="top", align_x=10, align_y=-10)

        self.key_layout = self.box.add(UIBoxLayout(space_between=20, align='left'))
        self.value_layout = self.box.add(UIBoxLayout(space_between=13, align='left'))

    def on_show_view(self):
        super().on_show_view()

        self.create_layouts()

        self.ui.push_handlers(self)

        self.back_button = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text='<--', style=button_style, width=100, height=50)
        self.back_button.on_click = lambda event: self.main_exit()
        self.top_box.add(self.back_button)

        self.display_categories()

        self.display_category(settings_start_category)

    def display_categories(self):
        for category in settings:
            category_button = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text=category, style=button_style, width=self.window.width / 10, height=50)

            if not category == "Credits":
                category_button.on_click = lambda event, category=category: self.display_category(category)
            else:
                category_button.on_click = lambda event: self.credits()

            self.top_box.add(category_button)

    def display_category(self, category):
        if hasattr(self, 'apply_button'):
            self.anchor.remove(self.apply_button)
            del self.apply_button

        if hasattr(self, 'credits_label'):
            self.anchor.remove(self.credits_label)
            del self.credits_label

        self.current_category = category

        self.key_layout.clear()
        self.value_layout.clear()

        for setting in settings[category]:
            label = arcade.gui.UILabel(text=setting, font_name="Roboto", font_size=28, text_color=arcade.color.WHITE )
            self.key_layout.add(label)

            setting_dict = settings[category][setting]

            if setting_dict['type'] == "option":
                dropdown = arcade.gui.UIDropdown(options=setting_dict['options'], width=200, height=50, default=self.settings_dict.get(setting_dict["config_key"], setting_dict["options"][0]), active_style=dropdown_style, dropdown_style=dropdown_style, primary_style=dropdown_style)
                dropdown.on_change = lambda _, setting=setting, dropdown=dropdown: self.update(setting, dropdown.value, "option")
                self.value_layout.add(dropdown)

            elif setting_dict['type'] == "bool":
                button_layout = self.value_layout.add(arcade.gui.UIBoxLayout(space_between=50, vertical=False))

                on_radiobutton = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text="ON", style=button_style, width=150, height=50)
                self.on_radiobuttons[setting] = on_radiobutton
                on_radiobutton.on_click = lambda _, setting=setting: self.update(setting, True, "bool")
                button_layout.add(on_radiobutton)

                off_radiobutton = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text="OFF", style=button_style, width=150, height=50)
                self.off_radiobuttons[setting] = off_radiobutton
                off_radiobutton.on_click = lambda _, setting=setting: self.update(setting, False, "bool")
                button_layout.add(off_radiobutton)

                if self.settings_dict.get(setting_dict["config_key"], setting_dict["default"]):
                    self.set_highlighted_style(on_radiobutton)
                    self.set_normal_style(off_radiobutton)
                else:
                    self.set_highlighted_style(off_radiobutton)
                    self.set_normal_style(on_radiobutton)

            elif setting_dict['type'] == "slider":
                if setting == "FPS Limit":
                    if self.settings_dict.get(setting_dict["config_key"]) == 0:
                        label_text = "FPS Limit: Disabled"
                    else:
                        label_text = f"FPS Limit: {self.settings_dict.get(setting_dict['config_key'], setting_dict['default'])}"
                else:
                    label_text = f"{setting}: {int(self.settings_dict.get(setting_dict['config_key'], setting_dict['default']))}"

                label.text = label_text

                self.slider_labels[setting] = label

                slider = arcade.gui.UISlider(width=400, height=50, value=self.settings_dict.get(setting_dict["config_key"], setting_dict["default"]), min_value=setting_dict['min'], max_value=setting_dict['max'], style=slider_style)
                slider.on_change = lambda _, setting=setting, slider=slider: self.update(setting, slider.value, "slider")

                self.sliders[setting] = slider
                self.value_layout.add(slider)

        self.apply_button = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text='Apply', style=button_style, width=200, height=100)
        self.apply_button.on_click = lambda event: self.apply_settings()
        self.anchor.add(self.apply_button, anchor_x="right", anchor_y="bottom", align_x=-10, align_y=10)

    def apply_settings(self):
        for config_key, value in self.modified_settings.items():
            self.settings_dict[config_key] = value

            if self.settings_dict['window_mode'] == "Fullscreen":
                self.window.set_fullscreen(True)
            else:
                self.window.set_fullscreen(False)
                width, height = map(int, self.settings_dict['resolution'].split('x'))
                self.window.set_size(width, height)

            if self.settings_dict['vsync']:
                self.window.set_vsync(True)
                display_mode = self.window.display.get_default_screen().get_mode()
                refresh_rate = display_mode.rate
                self.window.set_update_rate(1 / refresh_rate)
                self.window.set_draw_rate(1 / refresh_rate)

            elif not self.settings_dict['fps_limit'] == 0:
                self.window.set_vsync(False)
                self.window.set_update_rate(1 / self.settings_dict['fps_limit'])
                self.window.set_draw_rate(1 / self.settings_dict['fps_limit'])

            else:
                self.window.set_vsync(False)
                self.window.set_update_rate(1 / 99999999)
                self.window.set_draw_rate(1 / 99999999)

            if self.settings_dict['discord_rpc']:
                if isinstance(self.pypresence_client, FakePyPresence): # the user has enabled RPC in the settings in this session.
                    start_time = copy.deepcopy(self.pypresence_client.start_time)
                    self.pypresence_client.close()
                    del self.pypresence_client
                    try:
                        self.pypresence_client = pypresence.Presence(discord_presence_id)
                        self.pypresence_client.connect()
                        self.pypresence_client.update(state='In Settings', details='Modifying Settings', start=start_time)
                        self.pypresence_client.start_time = start_time
                    except:
                        self.pypresence_client = FakePyPresence()
                        self.pypresence_client.start_time = start_time
            else:
                if not isinstance(self.pypresence_client, FakePyPresence):
                    start_time = copy.deepcopy(self.pypresence_client.start_time)
                    self.pypresence_client.update()
                    self.pypresence_client.close()
                    del self.pypresence_client
                    self.pypresence_client = FakePyPresence()
                    self.pypresence_client.start_time = start_time

            self.ui_cleanup()

            self.ui = arcade.gui.UIManager()
            self.ui.enable()

            self.create_layouts()

            self.back_button = arcade.gui.UITextureButton(texture=button_texture, texture_hovered=button_hovered_texture, text='<--', style=button_style, width=100, height=50)
            self.back_button.on_click = lambda event: self.main_exit()
            self.top_box.add(self.back_button)

            self.display_categories()

        self.display_category(self.current_category)

        with open("settings.json", "w") as file:
            file.write(json.dumps(self.settings_dict, indent=4))

    def update(self, setting=None, button_state=None, setting_type="bool"):
        setting_dict = settings[self.current_category][setting]
        config_key = settings[self.current_category][setting]["config_key"]

        if setting_type == "option":
            self.modified_settings[config_key] = button_state

        elif setting_type == "bool":
            self.modified_settings[config_key] = button_state

            if button_state:
                self.set_highlighted_style(self.on_radiobuttons[setting])
                self.set_normal_style(self.off_radiobuttons[setting])
            else:
                self.set_highlighted_style(self.off_radiobuttons[setting])
                self.set_normal_style(self.on_radiobuttons[setting])

        elif setting_type == "slider":
            new_value = int(button_state)

            self.modified_settings[config_key] = new_value
            self.sliders[setting].value = new_value

            if setting == "FPS Limit":
                if new_value == 0:
                    label_text = "FPS Limit: Disabled"
                else:
                    label_text = f"FPS Limit: {str(new_value).rjust(8)}"
            else:
                label_text = f"{setting}: {str(new_value).rjust(8)}"

            self.slider_labels[setting].text = label_text

    def credits(self):
        if hasattr(self, 'apply_button'):
            self.anchor.remove(self.apply_button)
            del self.apply_button

        if hasattr(self, 'credits_label'):
            self.anchor.remove(self.credits_label)
            del self.credits_label

        self.key_layout.clear()
        self.value_layout.clear()

        with open('CREDITS', 'r') as file:
            text = file.read()

        if self.window.width == 3840:
            font_size = 30
        elif self.window.width == 2560:
            font_size = 20
        elif self.window.width == 1920:
            font_size = 17
        elif self.window.width >= 1440:
            font_size = 14
        else:
            font_size = 12

        self.credits_label = arcade.gui.UILabel(text=text, text_color=arcade.color.WHITE, font_name="Roboto", font_size=font_size, align="center", multiline=True)

        self.key_layout.add(self.credits_label)

    def set_highlighted_style(self, element):
        element.texture = button_hovered_texture
        element.texture_hovered = button_texture

    def set_normal_style(self, element):
        element.texture_hovered = button_hovered_texture
        element.texture = button_texture

    def main_exit(self):
        from wizard_vs_irs.menus.main import Main
        self.window.show_view(Main(self.pypresence_client, *self.args))

    def ui_cleanup(self):
        self.ui.clear()
        del self.ui



import arcade, arcade.gui, random, math, time, json, math

from wizard_vs_irs.utils.constants import (
    ABILITIES,
    ATTACK_INTERVAL_DECREASE_PER_LEVEL,
    BULLET_SPEED,
    HEALTH_INCREASE_PER_LEVEL,
    INVENTORY_ITEMS,
    INVENTORY_TRIGGER_KEYS,
    IRS_AGENT_SPAWN_INTERVAL,
    IRS_AGENT_TYPES,
    item_to_json_name,  # if it's a variable, better to keep original case
    PLAYER_INACCURACY_MAX,
    PLAYER_SPEED,
    SPAWN_INTERVAL_DECREASE_PER_LEVEL,
    SPEED_INCREASE_PER_LEVEL,
    TAX_EVASION_LEVELS,
    TAX_EVASION_NAMES,
    TAX_INCREASE_PER_LEVEL,
)

import wizard_vs_irs.utils.preload
from wizard_vs_irs.utils.preload import irs_agent_texture
from wizard_vs_irs.utils.preload import light_wizard_left_animation, light_wizard_right_animation, light_wizard_standing_animation, light_wizard_up_animation
from wizard_vs_irs.utils.preload import dark_wizard_left_animation, dark_wizard_right_animation, dark_wizard_standing_animation, dark_wizard_up_animation

from wizard_vs_irs.game.inventory import Inventory

class Bullet(arcade.Sprite):
    def __init__(self, radius, texture, x, y, direction):
        super().__init__(texture, center_x=x, center_y=y)
        self.radius = radius
        self.direction = direction
        self.speed = 0

    def move(self):
        self.position += self.direction * self.speed

class IRSAgent(arcade.Sprite):
    def __init__(self, x, y):
        super().__init__(irs_agent_texture, center_x=x, center_y=y, scale=1.25)

        self.speed, self.attack_speed, self.health, self.tax = 0, 0, 0, 0

        self.damaged = False
        self.last_damage = time.perf_counter()
        self.last_attack = time.perf_counter()

    def update(self):
        if self.damaged:
            if time.perf_counter() - self.last_damage >= 0.3:
                self.damaged = False

            self.color = arcade.color.RED
        else:
            self.color = (255, 255, 255, 255)

class Player(arcade.TextureAnimationSprite):
    def __init__(self, x, y, dark_mode_unlocked=False): # x, y here because we dont know window width and height
        super().__init__(animation=dark_wizard_standing_animation if dark_mode_unlocked else light_wizard_standing_animation, center_x=x, center_y=y, scale=1.5)

        self.direction = arcade.math.Vec2()

    def set_player_animation(self, animation):  # this is needed because the animation property will reset to the first frame, so animation doesnt work.
        self.animation = animation

class DamageNumberLabel(arcade.gui.UILabel):
    def __init__(self, x, y, damage):
        super().__init__(x=x, y=y, text=f"-{int(damage)}", text_color=arcade.color.RED)

        self.original_y = y
        self.finished = False

    def update(self):
        if self.center_y - self.original_y < 50:
            self.rect = self.rect.move(0, 2)
        else:
            self.finished = True

class Game(arcade.gui.UIView):
    def __init__(self, pypresence_client):
        super().__init__()

        self.pypresence_client = pypresence_client
        self.pypresence_client.update(state="Playing the game")

        with open("data.json", "r") as file: # no need for if, since Main already creates the file with default values.
            self.data = json.load(file)

        self.camera = arcade.Camera2D()

        self.camera_shake = arcade.camera.grips.ScreenShake2D(
            self.camera.view_data,
            max_amplitude=10.0,
            acceleration_duration=0.1,
            falloff_time=0.5,
            shake_frequency=10.0,
        )

        self.anchor = self.add_widget(arcade.gui.UIAnchorLayout(size_hint=(1, 1)))
        self.spritelist = arcade.SpriteList()

        self.irs_agents: list[IRSAgent] = []
        self.damage_numbers: list[arcade.gui.UILabel] = []
        self.last_irs_agent_spawn = time.perf_counter()
        self.last_mana = time.perf_counter()
        self.last_shoot = time.perf_counter()

        self.evaded_tax = 0
        self.high_score = self.data["high_score"]
        self.mana = 0
        self.tax_evasion_level = TAX_EVASION_NAMES[0]

        self.tax_shield = 0
        self.immobilize_irs = False
        self.last_immobilization = time.perf_counter()
        self.last_ability_timers = {}

        self.bullets: list[Bullet] = []
        self.player = Player(self.window.width / 2, self.window.height / 2, self.data["shop"].get("dark_mode_wizard", 0))
        self.spritelist.append(self.player)

        self.info_box = self.anchor.add(arcade.gui.UIBoxLayout(space_between=0, align="left"), anchor_x="left", anchor_y="top")
        self.evaded_tax_label = self.info_box.add(arcade.gui.UILabel(text="Evaded Tax: 0$", font_size=14))
        self.high_score_label = self.info_box.add(arcade.gui.UILabel(text=f"High Score: {self.high_score}$", font_size=14))
        self.mana_label = self.info_box.add(arcade.gui.UILabel(text="Mana: 0", font_size=14))
        self.tax_evasion_label = self.info_box.add(arcade.gui.UILabel(text=f"Tax Evasion Level: {self.tax_evasion_level}", font_size=14))
        
        self.tax_evasion_level_notice = self.anchor.add(arcade.gui.UILabel(text="Tax Evasion Level Increased to example", font_size=28), anchor_x="center", anchor_y="top")
        self.tax_evasion_level_notice.visible = False
        self.last_tax_evasion_notice = time.perf_counter()

        self.progress_bar = self.anchor.add(arcade.gui.UISlider(value=0, max_value=100, size_hint=(0.5, 0.15)), anchor_x="center", anchor_y="top")
        self.progress_bar._render_steps = lambda surface: None
        self.progress_bar._render_thumb = lambda surface: None
        self.progress_bar.on_event = lambda event: None

        self.ability_info_label = self.anchor.add(arcade.gui.UILabel(text=f"""Abilities:
Dash (tab): {ABILITIES['dash']} Mana
Tax Shield (t): {ABILITIES["tax_shield"]} Mana
Audit Bomb (b): {ABILITIES["audit_bomb"]} Mana
Freeze Audit (f): {ABILITIES["freeze_audit"]} Mana""", font_size=20, multiline=True), 
anchor_x="right", anchor_y="bottom", align_x=-5)

        self.inventory = self.anchor.add(Inventory(INVENTORY_ITEMS, self.window.width), anchor_x="left", anchor_y="bottom", align_x=self.window.width / 20)
        self.inventory.pay_tax_button.on_click = lambda event: self.pay_tax()

    def damage_irs_agent(self, irs_agent):
        irs_agent.damaged = True
        irs_agent.last_damage = time.perf_counter()
        
        item_list = INVENTORY_ITEMS[self.inventory.current_inventory_item]

        json_name = item_to_json_name[item_list[0]]

        damage = item_list[2] + (item_list[2] / 10 * self.data["shop"].get(f"{json_name}_dmg", 0))

        irs_agent.health -= damage

        self.damage_numbers.append(self.ui.add(DamageNumberLabel(irs_agent.left, irs_agent.top, damage)))

        if irs_agent.health <= 0:
            self.spritelist.remove(irs_agent)
            self.irs_agents.remove(irs_agent)
            self.evaded_tax += irs_agent.tax / 2
            self.update_evasion_level()

        self.camera_shake.start()

    def ability(self, ability):
        if self.mana >= ABILITIES[ability]:
            self.mana -= ABILITIES[ability]
            self.last_ability_timers[ability] = time.perf_counter()

            if ability == "dash":
                self.player.position += self.player.direction * (PLAYER_SPEED + self.data.get('shop', {}).get('player_speed', 0)) * 30
            elif ability == "tax_shield":
                self.tax_shield += 750
            elif ability == "audit_bomb":
                for irs_agent in self.irs_agents:
                    if arcade.math.Vec2(self.player.center_x, self.player.center_y).distance((irs_agent.center_x, irs_agent.center_y)) <= 250:
                        for i in range(3):
                            if irs_agent in self.irs_agents: # if they died the first or second time, they cant be damaged again
                                self.damage_irs_agent(irs_agent)
            elif ability == "freeze_audit":
                self.last_immobilization = time.perf_counter()
                self.immobilize_irs = True

    def spawn_bullet(self, direction):
        bullet = Bullet(INVENTORY_ITEMS[self.inventory.current_inventory_item][3], getattr(wizard_vs_irs.utils.preload, INVENTORY_ITEMS[self.inventory.current_inventory_item][4]), self.player.center_x, self.player.center_y, direction)
        bullet.speed = BULLET_SPEED + self.data.get('shop', {}).get("bullet_speed", 0)
        self.bullets.append(bullet)
        self.spritelist.append(bullet)

    def get_current_level_int(self):
        return TAX_EVASION_NAMES.index(self.tax_evasion_level)

    def update_evasion_level(self):
        before = self.get_current_level_int()

        if self.evaded_tax <= 0:
            self.tax_evasion_level = TAX_EVASION_NAMES[0]
        else:
            for tax_evasion_level, tax_evasion_min in TAX_EVASION_LEVELS.items():
                if self.evaded_tax >= tax_evasion_min:
                    self.tax_evasion_level = tax_evasion_level

        if before < self.get_current_level_int():
            self.tax_evasion_level_notice.text = f"Tax Evasion Level Increased to {self.tax_evasion_level}"
            self.tax_evasion_level_notice.visible = True
            self.last_tax_evasion_notice = time.perf_counter()
        elif before > self.get_current_level_int():
            self.tax_evasion_level_notice.text = f"Tax Evasion Level Decreased to {self.tax_evasion_level}"
            self.tax_evasion_level_notice.visible = True
            self.last_tax_evasion_notice = time.perf_counter()

        if not self.evaded_tax < 0:
            if not self.get_current_level_int() == len(TAX_EVASION_NAMES) - 1:
                self.progress_bar.value = ((self.evaded_tax - TAX_EVASION_LEVELS[self.tax_evasion_level]) / (TAX_EVASION_LEVELS[TAX_EVASION_NAMES[self.get_current_level_int() + 1]] - TAX_EVASION_LEVELS[self.tax_evasion_level])) * 100
            else:
                self.progress_bar.value = 100
        else:
            self.progress_bar.value = 0

        self.tax_evasion_label.text = f"Tax Evasion Level: {self.tax_evasion_level}"

    def pay_tax(self):
        if self.evaded_tax >= 1000:
            self.evaded_tax -= 1000
            self.update_evasion_level()

    def spawn_irs_agent(self):
        base_x = self.window.width / 2
        base_y = self.window.height / 2
        amount = self.window.width / 3

        angle = random.randint(0, 361)

        x = base_x + (math.cos(angle) * amount)
        y = base_y + (math.sin(angle) * amount)
        atk_speed, speed, health, tax = random.choice(IRS_AGENT_TYPES)

        agent = IRSAgent(x, y)
        agent.attack_speed = atk_speed - (ATTACK_INTERVAL_DECREASE_PER_LEVEL * self.get_current_level_int())
        agent.speed = speed + (SPEED_INCREASE_PER_LEVEL * self.get_current_level_int())
        agent.health = health + (HEALTH_INCREASE_PER_LEVEL * self.get_current_level_int())
        agent.tax = tax + (TAX_INCREASE_PER_LEVEL * self.get_current_level_int())
        
        self.irs_agents.append(agent)
        self.spritelist.append(agent)

    def on_update(self, delta_time):
        if self.immobilize_irs and time.perf_counter() - self.last_immobilization >= 4:
            self.immobilize_irs = False

        for damage_number_label in self.damage_numbers:
            damage_number_label.update()

            if damage_number_label.finished:
                self.damage_numbers.remove(damage_number_label)
                self.ui.remove(damage_number_label)
                
        self.camera_shake.update(delta_time)
        self.player.update_animation()

        if self.window.keyboard[arcade.key.W]:
            self.player.direction = arcade.math.Vec2(self.player.direction.x, 1)
            self.player.set_player_animation(dark_wizard_up_animation if self.data["shop"].get("dark_mode_wizard", False) else light_wizard_up_animation)
        elif self.window.keyboard[arcade.key.S]:
            self.player.direction = arcade.math.Vec2(self.player.direction.x, -1)
            self.player.set_player_animation(dark_wizard_standing_animation if self.data["shop"].get("dark_mode_wizard", False) else light_wizard_standing_animation)
        else:
            self.player.direction = arcade.math.Vec2(self.player.direction.x, 0)

        if self.window.keyboard[arcade.key.D]:
            self.player.direction = arcade.math.Vec2(1, self.player.direction.y)
            self.player.set_player_animation(dark_wizard_right_animation if self.data["shop"].get("dark_mode_wizard", False) else light_wizard_right_animation)
        elif self.window.keyboard[arcade.key.A]:
            self.player.direction = arcade.math.Vec2(-1, self.player.direction.y)
            self.player.set_player_animation(dark_wizard_left_animation if self.data["shop"].get("dark_mode_wizard", False) else light_wizard_left_animation)
        else:
            self.player.direction = arcade.math.Vec2(0, self.player.direction.y)

        self.player.position += self.player.direction * (PLAYER_SPEED + self.data.get('shop', {}).get('player_speed', 0))

        if self.player.center_x + self.player.width / 2 > self.window.width:
            self.player.center_x = self.window.width - self.player.width / 2
        elif self.player.center_x - self.player.width / 2 < 0:
            self.player.center_x = self.player.width / 2

        if self.player.center_y + self.player.height / 2 > self.window.height:
            self.player.center_y = self.window.height - self.player.height / 2
        elif self.player.center_y - self.player.height / 2 < 0:
            self.player.center_y = self.player.height / 2

        item_list = INVENTORY_ITEMS[self.inventory.current_inventory_item]

        json_name = item_to_json_name[item_list[0]]

        if time.perf_counter() - self.last_shoot >= item_list[1] - ((item_list[1] / 15) * self.data["shop"].get(f"{json_name}_atk_speed", 0)):
            self.last_shoot = time.perf_counter()
            
            mouse_pos = arcade.math.Vec2(
                self.window.mouse.data.get("x", 0),
                self.window.mouse.data.get("y", 0)
            )

            player_pos = arcade.math.Vec2(self.player.center_x, self.player.center_y)

            direction = (mouse_pos - player_pos).normalize()

            inaccuracy = random.randint(-(PLAYER_INACCURACY_MAX - self.data["shop"].get("inaccuracy_decrease", 0)), (PLAYER_INACCURACY_MAX - self.data["shop"].get("inaccuracy_decrease", 0)))
            self.spawn_bullet(direction.rotate(math.radians(inaccuracy)))

        if self.tax_evasion_level_notice.visible and time.perf_counter() - self.last_tax_evasion_notice >= 2.5:
            self.tax_evasion_level_notice.visible = False

        if time.perf_counter() - self.last_mana >= 0.5:
            self.last_mana = time.perf_counter()
            
            self.mana += 5
            
            self.mana_label.text = f"Mana: {self.mana}"
        
        if not self.immobilize_irs:
            for irs_agent in self.irs_agents:
                irs_agent.update()

                wizard_pos_vec = arcade.math.Vec2(self.player.center_x, self.player.center_y)

                if wizard_pos_vec.distance(irs_agent.position) <= self.player.width / 2:
                    if time.perf_counter() - irs_agent.last_attack >= irs_agent.attack_speed:
                        irs_agent.last_attack = time.perf_counter()

                        self.camera_shake.start()

                        if not self.tax_shield > 0:
                            self.evaded_tax -= irs_agent.tax
                        else:
                            if irs_agent.tax > self.tax_shield:
                                self.evaded_tax -= irs_agent.tax - self.tax_shield
                                self.tax_shield = 0
                            else:
                                self.tax_shield -= irs_agent.tax

                        self.damage_numbers.append(self.ui.add(DamageNumberLabel(self.player.left, self.player.top, irs_agent.tax)))
        
                        self.update_evasion_level()
                else:
                    direction = (wizard_pos_vec - irs_agent.position).normalize()
                    irs_agent.angle = -math.degrees(direction.heading())
                    irs_agent.position += direction * irs_agent.speed

        for bullet in self.bullets:
            bullet.move()

            hit = False

            for irs_agent in self.irs_agents:
                if arcade.math.Vec2(bullet.center_x, bullet.center_y).distance((irs_agent.center_x, irs_agent.center_y)) <= (irs_agent.width / 2 + bullet.radius):
                    self.damage_irs_agent(irs_agent)
                    damage = item_list[2] + (item_list[2] / 10 * self.data["shop"].get(f"{json_name}_dmg", 0))
                    irs_agent.position += bullet.direction * damage * 1.5
                    hit = True

            if hit or bullet.center_x + bullet.radius / 2 > self.window.width or bullet.center_x - bullet.radius / 2 < 0 or bullet.center_y + bullet.radius / 2 > self.window.height or bullet.center_y - bullet.height / 2 < 0:
                self.spritelist.remove(bullet)
                self.bullets.remove(bullet)

        if time.perf_counter() - self.last_irs_agent_spawn >= IRS_AGENT_SPAWN_INTERVAL - (SPAWN_INTERVAL_DECREASE_PER_LEVEL * self.get_current_level_int()):
            self.last_irs_agent_spawn = time.perf_counter()

            self.spawn_irs_agent()

        if self.evaded_tax >= 0:
            self.evaded_tax_label.text = f"Evaded Tax: {int(self.evaded_tax)}$"
        else:
            self.evaded_tax_label.text = f"Tax Debt: {int(abs(self.evaded_tax))}$"

        if self.evaded_tax > self.high_score:
            self.high_score = self.evaded_tax
            self.high_score_label.text = f"High Score: {int(self.high_score)}$"

    def on_key_press(self, symbol, modifiers):
        if symbol == arcade.key.ESCAPE:
            self.data["high_score"] = int(self.high_score)
            self.data["evaded_tax"] += int(self.evaded_tax)
            with open("data.json", "w") as file:
                file.write(json.dumps(self.data, indent=4))

            from wizard_vs_irs.menus.main import Main
            self.window.show_view(Main(self.pypresence_client))
        elif symbol in INVENTORY_TRIGGER_KEYS:
            self.inventory.select_item(int(chr(symbol)) - 1)
        elif symbol == arcade.key.P:
            self.pay_tax()
        elif symbol == arcade.key.TAB:
            self.ability("dash")
        elif symbol == arcade.key.T:
            self.ability("tax_shield")
        elif symbol == arcade.key.B:
            self.ability("audit_bomb")
        elif symbol == arcade.key.F:
            self.ability("freeze_audit")

    def on_resize(self, width: int, height: int):
        super().on_resize(width, height)
        self.camera.match_window()

    def on_draw(self):
        self.window.clear()

        self.camera_shake.update_camera()
        self.camera.use()
        self.spritelist.draw()
        self.camera_shake.readjust_camera()

        self.ui.draw() # draw after, so UI is on top


import arcade, arcade.gui

from wizard_vs_irs.utils.constants import button_style

class Inventory(arcade.gui.UIBoxLayout):
    def __init__(self, items, window_width):
        super().__init__(size_hint=(0.5, 0.1), vertical=False, space_between=5)
        self.items = items
        self.current_inventory_item = 0

        self.inventory_buttons: list[arcade.gui.UITextureButton] = []

        for n, item in enumerate(items):
            self.inventory_buttons.append(self.add(arcade.gui.UIFlatButton(text=f"{item[0]} ({n + 1})", style=button_style, width=(window_width * 0.5) / len(items) + 1)))

        self.pay_tax_button = self.add(arcade.gui.UIFlatButton(text="Pay Tax (1000$)", style=button_style, width=(window_width * 0.5) / len(items) + 1))
        self.pay_tax_button.style["normal"]["bg"] = arcade.color.GRAY
        self.pay_tax_button.style["normal"]["bg"] = arcade.color.GRAY

        self.update_selection()

    def update_selection(self):
        for n in range(len(self.items)):
            if n == self.current_inventory_item:
                self.inventory_buttons[n].style["normal"] = arcade.gui.UIFlatButton.UIStyle(bg=arcade.color.WHITE, font_color=arcade.color.BLACK)
            else:
                self.inventory_buttons[n].style["normal"] = arcade.gui.UIFlatButton.UIStyle(bg=arcade.color.GRAY, font_color=arcade.color.BLACK)

            self.inventory_buttons[n]._requires_render = True

    def select_item(self, number):
        self.current_inventory_item = number
        self.update_selection()


import arcade.color, arcade.key
from arcade.types import Color
from arcade.gui.widgets.buttons import UITextureButtonStyle, UIFlatButtonStyle
from arcade.gui.widgets.slider import UISliderStyle

menu_background_color = (30, 30, 47)
log_dir = 'logs'
discord_presence_id = 1424784736726945915

IRS_AGENT_SPAWN_INTERVAL = 0.8

IRS_AGENT_TYPES = [
    (1, 2.25, 15, 200), # Normal
    (0.9, 3.5, 8, 150), # Speedrunner
    (1.3, 1.5, 35, 350) # Auditor
]

SPEED_INCREASE_PER_LEVEL = 1 / 5
SPAWN_INTERVAL_DECREASE_PER_LEVEL = 1 / 20
HEALTH_INCREASE_PER_LEVEL = 1 / 7.5
TAX_INCREASE_PER_LEVEL = 1 / 3
ATTACK_INTERVAL_DECREASE_PER_LEVEL = 1 / 10

TAX_EVASION_LEVELS = {
    "Compliant Citizen": 0,
    "Minor Mistake": 1000,            
    "Mildly Suspicious": 2500,
    "Suspicious": 5000,
    "Under Review": 7500,
    "Investigated": 10000,
    "Flagged": 13500,
    "Audited": 25000,
    "Seized Assets": 40000,
    "Criminal Case": 70000,
    "International Watchlist": 125000,
    "Most Wanted": 250000,
    "Legendary": 500000,
    "THE ONE AND ONLY TAX EVADER": 1000000
}
TAX_EVASION_NAMES = list(TAX_EVASION_LEVELS.keys())

BULLET_SPEED = 8
PLAYER_INACCURACY_MAX = 10
PLAYER_SPEED = 4

# name, json_key, max_count, upgrade_cost
SHOP_ITEMS = [
    ["Fireball DMG", "fb_dmg", 999, 10000],
    ["Fireball ATK Speed", "fb_atk_speed", 10, 30000],
    ["Ball of Lightning DMG", "lb_dmg", 999, 10000],
    ["Ball of Lightning ATK Speed", "lb_atk_speed", 10, 30000],
    ["Ice Blast DMG", "ib_dmg", 999, 10000],
    ["Ice Blast ATK Speed", "ib_atk_speed", 10, 30000],
    ["Inaccuracy Decrease", "inaccuracy_decrease", 10, 25000],
    ["Player Speed", "player_speed", 15, 20000],
    ["Bullet Speed", "bullet_speed", 999, 20000],
    ["Dark Mode Wizard", "dark_mode_wizard", 1, 1000000]
]

INVENTORY_ITEMS = [
    ["Fireball", 0.25, 10, 10, "fireball_texture"],
    ["Ball of Lightning", 0.45, 20, 20, "lightning_bolt_texture"],
    ["Ice Blast", 0.15, 5, 7.5, "ice_blast_texture"],
]

ABILITIES = {
    "dash": 20,
    "tax_shield": 50,
    "audit_bomb": 100,
    "freeze_audit": 150
}

item_to_json_name = {
    "Fireball": "fb",
    "Ball of Lightning": "bl",
    "Ice Blast": "ib"
}

INVENTORY_TRIGGER_KEYS = [getattr(arcade.key, f"KEY_{n+1}") for n in range(len(INVENTORY_ITEMS))]

button_style = {'normal': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK), 'hover': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK),
                'press': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK), 'disabled': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK)}
big_button_style = {'normal': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, font_size=26), 'hover': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, font_size=26),
                'press': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, font_size=26), 'disabled': UITextureButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, font_size=26)}

dropdown_style = {'normal': UIFlatButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, bg=Color(128, 128, 128)), 'hover': UIFlatButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, bg=Color(49, 154, 54)),
                  'press': UIFlatButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, bg=Color(128, 128, 128)), 'disabled': UIFlatButtonStyle(font_name="Roboto", font_color=arcade.color.BLACK, bg=Color(128, 128, 128))}

slider_default_style = UISliderStyle(bg=Color(128, 128, 128), unfilled_track=Color(128, 128, 128), filled_track=Color(49, 154, 54))
slider_hover_style = UISliderStyle(bg=Color(49, 154, 54), unfilled_track=Color(128, 128, 128), filled_track=Color(49, 154, 54))

slider_style = {'normal': slider_default_style, 'hover': slider_hover_style, 'press': slider_hover_style, 'disabled': slider_default_style}

settings = {
    "Graphics": {
        "Window Mode": {"type": "option", "options": ["Windowed", "Fullscreen", "Borderless"], "config_key": "window_mode", "default": "Windowed"},
        "Resolution": {"type": "option", "options": ["1366x768", "1440x900", "1600x900", "1920x1080", "2560x1440", "3840x2160"], "config_key": "resolution"},
        "Anti-Aliasing": {"type": "option", "options": ["None", "2x MSAA", "4x MSAA", "8x MSAA", "16x MSAA"], "config_key": "anti_aliasing", "default": "4x MSAA"},
        "VSync": {"type": "bool", "config_key": "vsync", "default": True},
        "FPS Limit": {"type": "slider", "min": 0, "max": 480, "config_key": "fps_limit", "default": 60},
    },
    "Sound": {
        "Music": {"type": "bool", "config_key": "music", "default": True},
        "SFX": {"type": "bool", "config_key": "sfx", "default": True},
        "Music Volume": {"type": "slider", "min": 0, "max": 100, "config_key": "music_volume", "default": 50},
        "SFX Volume": {"type": "slider", "min": 0, "max": 100, "config_key": "sfx_volume", "default": 50},
    },
    "Miscellaneous": {
        "Discord RPC": {"type": "bool", "config_key": "discord_rpc", "default": True},
    },
    "Credits": {}
}
settings_start_category = "Graphics"



import arcade.gui, arcade

button_texture = arcade.gui.NinePatchTexture(64 // 4, 64 // 4, 64 // 4, 64 // 4, arcade.load_texture("assets/graphics/button.png"))
button_hovered_texture = arcade.gui.NinePatchTexture(64 // 4, 64 // 4, 64 // 4, 64 // 4, arcade.load_texture("assets/graphics/button_hovered.png"))

light_wizard_spritesheet = arcade.load_spritesheet("assets/graphics/mage-light.png").get_texture_grid((48, 64), 3, 12)
light_wizard_up_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in light_wizard_spritesheet[0:2]])
light_wizard_right_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in light_wizard_spritesheet[3:5]])
light_wizard_standing_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in light_wizard_spritesheet[6:8]])
light_wizard_left_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in light_wizard_spritesheet[9:11]])

dark_wizard_spritesheet = arcade.load_spritesheet("assets/graphics/mage-dark.png").get_texture_grid((48, 64), 3, 12)
dark_wizard_up_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in dark_wizard_spritesheet[0:2]])
dark_wizard_right_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in dark_wizard_spritesheet[3:5]])
dark_wizard_standing_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in dark_wizard_spritesheet[6:8]])
dark_wizard_left_animation = arcade.TextureAnimation([arcade.TextureKeyframe(texture, 300) for texture in dark_wizard_spritesheet[9:11]])

fireball_texture = arcade.make_circle_texture(10, arcade.color.RED)
lightning_bolt_texture = arcade.make_circle_texture(20, arcade.color.BLUE)
ice_blast_texture = arcade.make_circle_texture(20, arcade.color.ICEBERG)

irs_agent_texture = arcade.load_texture("assets/graphics/irs_agent.png")